@@ -3,7 +3,7 @@
 The uWSGI Emperor
 
 */
-#include <uwsgi.h>
+#include "uwsgi.h"
 
 
 extern struct uwsgi_server uwsgi;
@@ -722,7 +722,7 @@ void emperor_back_to_ondemand(struct uwsgi_instance *c_ui) {
 	// remove uWSGI instance
 
 	if (c_ui->pid != -1) {
-		if (write(c_ui->pipe[0], "\0", 1) != 1) {
+		if (write(c_ui->pipe[0], uwsgi.emperor_graceful_shutdown ? "\2" : "\0", 1) != 1) {
 			uwsgi_error("emperor_stop()/write()");
 		}
 	}
@@ -739,7 +739,7 @@ void emperor_stop(struct uwsgi_instance *c_ui) {
 	// remove uWSGI instance
 
 	if (c_ui->pid != -1) {
-		if (write(c_ui->pipe[0], "\0", 1) != 1) {
+		if (write(c_ui->pipe[0], uwsgi.emperor_graceful_shutdown ? "\2" : "\0", 1) != 1) {
 			uwsgi_error("emperor_stop()/write()");
 		}
 	}
@@ -2410,8 +2410,9 @@ void uwsgi_master_manage_emperor() {
 		if (byte == 0) {
 			uwsgi_hooks_run(uwsgi.hook_emperor_stop, "emperor-stop", 0);
 			close(uwsgi.emperor_fd);
-			if (!uwsgi.status.brutally_reloading)
+			if (!uwsgi.status.brutally_reloading && !uwsgi.status.brutally_destroying) {
 				kill_them_all(0);
+			}
 		}
 		// reload me
 		else if (byte == 1) {
@@ -2422,6 +2423,14 @@ void uwsgi_master_manage_emperor() {
 			grace_them_all(0);
 			uwsgi_unblock_signal(SIGHUP);
 		}
+		// remove me gracefully
+		else if (byte == 2) {
+			uwsgi_hooks_run(uwsgi.hook_emperor_stop, "emperor-stop", 0);
+			close(uwsgi.emperor_fd);
+			if (!uwsgi.status.brutally_reloading && !uwsgi.status.brutally_destroying) {
+				gracefully_kill_them_all(0);
+			}
+		}
 	}
 #ifdef UWSGI_EVENT_USE_PORT
         // special cose for port event system
