@@ -22,6 +22,8 @@ void uwsgi_xml_config(char *filename, struct wsgi_request *wsgi_req, char *magic
 	char *xml_content;
 	size_t xml_size = 0;
 
+	LIBXML_TEST_VERSION
+
 	if (uwsgi_check_scheme(filename)) {
 		colon = uwsgi_get_last_char(filename, '/');
 		colon = uwsgi_get_last_char(colon, ':');
@@ -151,6 +153,8 @@ void uwsgi_xml_config(char *filename, struct wsgi_request *wsgi_req, char *magic
 	}
 	/* We can safely free resources */
 
+	if (colon) colon[0] = ':';
+
 	xmlFreeDoc(doc);
 	xmlCleanupParser();
 
@@ -267,6 +271,7 @@ void uwsgi_xml_config(char *filename, struct wsgi_request *wsgi_req, char *magic
 
 	} while (!done);
 
+	if (colon) colon[0] = ':';
 
 	// we can safely free, as we have a copy of datas
 	XML_ParserFree(parser);
