@@ -764,13 +764,12 @@ static int uwsgi_lua_request(struct wsgi_request *wsgi_req) {
 	lua_newtable(L);
 	lua_pushstring(L, "");
 	lua_setfield(L, -2, "CONTENT_TYPE");
-	for(i=0;i<wsgi_req->var_cnt;i++) {
+	for(i=0;i<wsgi_req->var_cnt;i+=2) {
 		lua_pushlstring(L, (char *)wsgi_req->hvec[i+1].iov_base, wsgi_req->hvec[i+1].iov_len);
 		// transform it in a valid c string TODO this is ugly
 		ptrbuf = wsgi_req->hvec[i].iov_base+wsgi_req->hvec[i].iov_len;
 		*ptrbuf = 0;
 		lua_setfield(L, -2, (char *)wsgi_req->hvec[i].iov_base);
-		i++;
 	}
 
 
@@ -811,8 +810,25 @@ static int uwsgi_lua_request(struct wsgi_request *wsgi_req) {
 	lua_pushnil(L);
 	while(lua_next(L, -3) != 0) {
 		http = lua_tolstring(L, -2, &slen);
-		http2 = lua_tolstring(L, -1, &slen2);
-		uwsgi_response_add_header(wsgi_req, (char *) http, slen, (char *) http2, slen2);
+
+		if (lua_type(L, -1) == LUA_TTABLE) {
+			for (i = 1; /*empty*/ ; ++i) {
+				lua_rawgeti(L, -1, i);
+
+				if (lua_isnil(L, -1)) {
+					lua_pop(L, 1);
+					break;
+				}
+
+				http2 = lua_tolstring(L, -1, &slen2);
+				uwsgi_response_add_header(wsgi_req, (char *) http, slen, (char *) http2, slen2);
+				lua_pop(L, 1);
+			}
+		}
+		else {
+			http2 = lua_tolstring(L, -1, &slen2);
+			uwsgi_response_add_header(wsgi_req, (char *) http, slen, (char *) http2, slen2);
+		}
 		lua_pop(L, 1);
 	}
 
