@@ -1,12 +1,12 @@
 #include <uwsgi.h>
-#if defined(__linux__) || defined(__GNU_kFreeBSD__)
+#if defined(__linux__) || defined(__GNU_kFreeBSD__) || defined(__HURD__)
 #include <pty.h>
 #elif defined(__APPLE__) || defined(__OpenBSD__) || defined(__NetBSD__)
 #include <util.h>
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__DragonFly__)
 #include <libutil.h>
 #endif
-#ifndef __FreeBSD__
+#if !defined(__FreeBSD__) && !defined(__DragonFly__)
 #include <utmp.h>
 #endif
 
@@ -292,7 +292,7 @@ static int uwsgi_pty_client() {
         	uwsgi.restore_tc = 1;
         }
 
-	upty.server_fd = uwsgi_connect(remote, uwsgi.shared->options[UWSGI_OPTION_SOCKET_TIMEOUT], 0);
+	upty.server_fd = uwsgi_connect(remote, uwsgi.socket_timeout, 0);
 	if (upty.server_fd < 0) {
 		uwsgi_error("uwsgi_pty_client()/connect()");
 		exit(1);
