@@ -93,7 +93,11 @@ ssize_t uwsgi_syslog_logger(struct uwsgi_logger *ul, char *message, size_t len)
 		ul->configured = 1;
 	}
 
+#ifdef __APPLE__
+	syslog(LOG_NOTICE, "%.*s", (int) len, message);
+#else
 	syslog(LOG_INFO, "%.*s", (int) len, message);
+#endif
 	return 0;
 
 }
