@@ -2,7 +2,7 @@ import os
 
 NAME='xslt'
 CFLAGS = os.popen('xslt-config --cflags').read().rstrip().split()
-LDFLAGS = os.popen('xslt-config --libs').read().rstrip().split()
-LIBS = []
+LDFLAGS = []
+LIBS = os.popen('xslt-config --libs').read().rstrip().split()
 
 GCC_LIST = ['xslt']
